import requests

from .builder import AmazonRequestBuilder
from .response import (
    AmazonItemSearchResponse,
    AmazonItemLookupResponse,
    AmazonSimilarityLookupResponse,
)


class AmazonProductAPI(object):
    def __init__(self, access_key, secret_key, associate_tag):
        self.access_key = access_key
        self.secret_key = secret_key
        self.associate_tag = associate_tag
        self.request_builder = AmazonRequestBuilder(access_key, secret_key, associate_tag)

    def _base_params(self, search_index=None, response_groups=None, parameters=None):
        if parameters is None:
            parameters = {}
        default_params = {
            'Service': 'AWSECommerceService',
            'AWSAccessKeyId': self.access_key,
            'AssociateTag': self.associate_tag,
            'ResponseGroup': 'Images,ItemAttributes',
            'Version': '2013-08-01'
        }
        if response_groups:
            default_params['ResponseGroup'] = ','.join(response_groups)
        if search_index:
            default_params['SearchIndex'] = search_index
        parameters.update(default_params)
        return parameters

    def item_search(self, search_index, keywords=None, page=None, response_groups=None, parameters=None):
        params = self._base_params(search_index, response_groups, parameters)
        params['Operation'] = 'ItemSearch'
        if keywords:
            params['Keywords'] = ','.join(keywords)
        if page:
            params['ItemPage'] = page
        req_url = self.request_builder.build_request_url(params)
        response = self._make_get_request(req_url, AmazonItemSearchResponse)
        return response

    def item_lookup(self, item_id, id_type='ASIN', search_index=None, response_groups=None, parameters=None):
        params = self._base_params(search_index, response_groups, parameters)
        params['Operation'] = 'ItemLookup'
        params['ItemId'] = item_id
        params['IdType'] = id_type
        req_url = self.request_builder.build_request_url(params)
        response = self._make_get_request(req_url, AmazonItemLookupResponse)
        return response

    def similarity_lookup(self, asins, response_groups=None, parameters=None):
        params = self._base_params(response_groups=response_groups, parameters=parameters)
        if not isinstance(asins, (list,)):
            asins = [asins]
        params['Operation'] = 'SimilarityLookup'
        params['ItemId'] = ','.join(asins)
        req_url = self.request_builder.build_request_url(params)
        response = self._make_get_request(req_url, AmazonSimilarityLookupResponse)
        return response

    def _make_get_request(self, req_url, response_class):
        req = requests.get(req_url)
        response = response_class(req)
        return response



from datetime import datetime
import urllib.parse
import hmac
import hashlib
import base64


class AmazonRequestBuilder(object):
    request_protocol = 'https'
    api_host = 'webservices.amazon.com'
    api_uri = '/onca/xml'

    def __init__(self, access_key, secret_key, associate_tag):
        self.access_key = access_key
        self.secret_key = secret_key
        self.associate_tag = associate_tag

    @property
    def current_timestamp(self):
        return datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%S.000Z')

    def build_request_url(self, params, override_timestamp=None, override_protocol=None):
        # Reference - https://docs.aws.amazon.com/AWSECommerceService/latest/DG/rest-signature.html
        params['Timestamp'] = override_timestamp or self.current_timestamp
        request_querystring = self._build_querystring(params)
        req_url = '{}://{}{}?{}'.format(override_protocol or self.request_protocol, self.api_host, self.api_uri, request_querystring)
        return req_url

    def _build_querystring(self, params):
        req_qs = self._encode_querystring(params)
        req_qs = self._sort_querystring(req_qs)
        req_qs = self._add_request_signature(req_qs)
        return req_qs

    def _sort_querystring(self, querystring):
        split_qs = querystring.split('&')
        split_qs.sort()
        qs = '&'.join(split_qs)
        return qs

    def _encode_querystring(self, params):
        request_querystring = urllib.parse.urlencode(params, quote_via=urllib.parse.quote)
        return request_querystring

    def _add_request_signature(self, querystring):
        string_to_sign = 'GET\n{}\n{}\n{}'.format(self.api_host, self.api_uri, querystring)
        signature = self._hash_signature(string_to_sign)
        encoded_signature = ''.join(
            [urllib.parse.quote(char) if char in ('+', '=') else char for char in signature])
        querystring_with_signature = '{}&Signature={}'.format(querystring, encoded_signature)
        return querystring_with_signature

    def _hash_signature(self, string):
        binary_key = bytes(self.secret_key, encoding='utf-8')
        hash = hmac.new(binary_key, string.encode('utf-8'), digestmod=hashlib.sha256)
        digest = hash.digest()
        encoded_digest = base64.b64encode(digest).decode()
        return encoded_digest


import json
import xmltodict


class AmazonProductAPIResponse(object):
    response_obj_name = None
    response_error_name = None

    def __init__(self, response_obj):
        self.status_code = response_obj.status_code
        self.raw_response_content = response_obj.content
        self.data = self._parse_data()
        self.headers = response_obj.headers

    def _parse_data(self):
        data_dict = json.loads(json.dumps(xmltodict.parse(self.raw_response_content, xml_attribs=False)))
        return data_dict

    @property
    def is_error(self):
        return self.status_code >= 400 or not self.data[self.response_obj_name]['Items']['Request']['IsValid'] == 'True'

    @property
    def error_msg(self):
        if self.is_error:
            return self._error_attrib('Message')

    @property
    def error_code(self):
        if self.is_error:
            return self._error_attrib('Code')

    def _error_attrib(self, attrib):
        if self.status_code >= 400:
            return self.data[self.response_error_name]['Error'][attrib]
        return self.data[self.response_obj_name]['Items']['Request']['Errors']['Error'][attrib]


class AmazonItemSearchResponse(AmazonProductAPIResponse):
    response_obj_name = 'ItemSearchResponse'
    response_error_name = 'ItemSearchErrorResponse'


class AmazonItemLookupResponse(AmazonProductAPIResponse):
    response_obj_name = 'ItemLookupResponse'
    response_error_name = 'ItemLookupErrorResponse'


class AmazonSimilarityLookupResponse(AmazonProductAPIResponse):
    response_obj_name = 'SimilarityLookupResponse'
    response_error_name = 'SimilarityLookupErrorResponse'



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="orangearrow",
    description="A Python Library for interacting with the Amazon Product Advertising API.",
    long_description=long_description,
    long_description_content_type='text/markdown',
    version="v0.9.64",
    author='Joe Wilson',
    license='Apache2',
    url="https://github.com/wreddily/orangearrow",
    packages=['orangearrow'],
    python_requires='>=3.4',
    install_requires=[
        "requests>=2.20.0",
        "xmltodict>=0.10.2",
    ],
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ]
)
